##----------------------------------------------------
##   R script to perform Functional PCA on NDVI data
##----------------------------------------------------

## Load NDVI data
setwd("/Users/mark.hebblewhite/Documents/MyDocuments/UniversityofMontana/GraduateStudents/MarkHurley/manuscripts/PTRSBpaper/FinalSubmission")
ndvi <- read.table("FinalTable_original.csv", h=T, sep = ",")
head(ndvi)
## In the ndvi object, we have weekly NDVI measures but the starting date varies among series. To solve this problem, we perform linear interpolation:

out <- matrix(nrow = nrow(ndvi), ncol = 52)
for(i in 1:nrow(ndvi)){
  y <- ndvi[i, 4:55]
  x <- seq(ndvi[i, 3], by = 7, length = sum(!is.na(y)))
  out[i, ] <- approx(x=x, y=y,
                     xout = seq(4, by=7, length=52),
                     method="linear")$y
  }

dim(out)
code <- paste(ndvi$Year, ndvi$Unit, sep =".")
rownames(out) <- code

## Keep only the april-december period
out <- out[, 14:49]
dim(out)
head(out)

##----------------------------
##     Functional analysis
##----------------------------

## we need to transpose the data table
out2 <- t(out)

## create the B-splines basis
date <- seq(4, by=7, length=52)[14:49]

install.packages(fda)
library(fda)
bas <- create.bspline.basis(range(date), breaks = date, norder = 4)
plot(bas)

## compute smoothed curves
fdPar.obj <- fdPar(bas, lambda = 1)
thebasis <- smooth.basis(date, as.matrix(out2), fdPar.obj, fdnames = list(date = row.names(out2), id = colnames(out2), "NDVI"))

## plot data and smoothed curves
par(mfrow = c(12, 5), mar = rep(0.1, 4))
for (i in 1:ncol(out2)) {
    plot(date, out2[, i], type = "n", axes = FALSE)
    lines(thebasis$fd[i], col = "grey", lwd = 1)
    points(date, out2[, i], pch = 20, cex = 0.5, col = "blue")
    text(max(date), 0.9 * max(out2[, i]), labels = names(out2)[i], cex = 1.8, pos = 2)
    box()
}
par(mfrow = c(1, 1))

## The smoothed curves and the average Idaho curve (in bold)
plot(thebasis$fd)
lines(mean(thebasis$fd), lwd = 4)

## Compute functional PCA
pc <- pca.fd(thebasis$fd, center=TRUE)

## Barplot of eigenvalues
barplot(pc$values)

## Plot harmonics (i.e. eigenfunctions)
par(mfrow = c(2, 1))
plot(pc)

##---------------------
## K-means partitioning
##---------------------

## select the best number of groups
library(vegan)
casc <- cascadeKM(pc$scores, 2, 7)
plot(casc)

## partition the individuals in 5 groups
km1 <- kmeans(pc$scores[, 1:2], 5)
clust <- as.factor(km1$cluster)

## optional: we reorder levels to obtain the same plot than in the paper
clust <- reorder(clust, -pc$scores[,1])
levels(clust) <- 1:5

## assign a col to each cluster
library(RColorBrewer)
thecol <- brewer.pal(5, "Paired")

## plot the clusters and individuals on FPCA factorial plane
library(ade4)
s.class(pc$scores[, 1:2], clust, col = thecol)

## draw the curves for each cluster
## (average curve of the cluster in bold, Idaho average in black)

par(mfrow = c(3, 2), mar = rep(2, 4))

for (i in 1:5) {
    plot(thebasis$fd[km1$cluster == i], col = thecol[i], lwd = 1, lty = 3, ylim = c(110, 175))
    lines(mean(thebasis$fd[km1$cluster == i]), col = thecol[i], lwd = 2)
    lines(mean(thebasis$fd))
}



## ---------------------------------------------------
## Percentage of variation explained by space and time
## ---------------------------------------------------
GMU <- as.factor(ndvi$Unit)
year <- as.factor(ndvi$Year)


pca1 <- dudi.pca(pc$scores, scale = FALSE, scannf = FALSE)
bet.GMU <- bca(pca1, GMU, scannf = FALSE)
bet.year <- bca(pca1, year, scannf = FALSE)

variation <- c(sum(pca1$eig), sum(bet.GMU$eig), sum(bet.year$eig))
variation <- c(variation, variation[1]-sum(variation[2:3]))
names(variation) <- c("total", "GMU", "year", "residual")
variation/variation[1]
